<?php
Namespace App\Model\Admin;

use Symfony\Component\Translation\TranslatorInterface;
use App\Utils\Utils;

/**
 * Provides admin functions for Homegear module management
 *
 */
class HomegearModule
{
    public $hg;
    private $familyConfig = '/etc/homegear/families/';
    private $translator;
    private $utils;

    public function __construct(TranslatorInterface $translator, Utils $utils) {
        $this->hg = new \Homegear\Homegear;
        $this->translator = $translator;
        $this->utils = $utils;
    }

    /**
     * List all installed modules
     * 
     * @return array()
     */
    public function listInstalled() {
        if (class_exists('App\Model\Admin\SmarthaLicense')) {
            $myLic = new SmarthaLicense();
            $modules = $myLic->listInstalled();
        } else {
            $modules = $this->hg->listModules();
            $licenses = $this->hg->getLicenseStates(hexdec('0x7000'));
            // Not checking the var throws a nasty 500 if you dont have easyled installed
            if ($licenses !== false) {
                foreach ($modules as $k => $v) {
                    // don't check misc here
                    if ($v['FAMILY_ID'] == 254) { continue; }
                    $lic = array_search($v['FAMILY_ID'], array_column($licenses,'FAMILY_ID'));
                    if ($lic !== false) {
                        $modules[$k]['license'] = '1';
                        if (strstr($licenses[$lic]['LICENSE_KEY'], 'trial')) {
                            $modules[$k]['isTrial'] = date('d.m.Y', strtotime('+30 days', (int)substr($licenses[$lic]['LICENSE_KEY'],5,10)));
                        }
                        $modules[$k]['licenseKey'] = $licenses[$lic]['LICENSE_KEY'];
                    }
                }
            }
        }
        usort($modules, function($a, $b) { return ucfirst($a['FAMILY_NAME']) <=> ucfirst($b['FAMILY_NAME']); });
        return $modules;
    }

    /**
     * List all available modules
     * 
     * @return array()
     */
    public function listAvailable()
    {
        // get the list of installable packages
        $modules = array();
        if (file_exists('/etc/homegear/smartha-management.conf')) {
            $packagesWhitelist = shell_exec("grep -i packageswhitelist /etc/homegear/smartha-management.conf");
        } else {
            $packagesWhitelist = shell_exec("grep -i packageswhitelist /etc/homegear/management.conf");
        }
        if (isset($packagesWhitelist) && $packagesWhitelist != '') {
            $packagesWhitelist = preg_replace('/^\w+ = (.*)\n$/', '${1}', $packagesWhitelist);
            if (strpos($packagesWhitelist,"#") === false) {
                $packagesWhitelist = explode(' ',$packagesWhitelist);
            }
        }
        if (isset($packagesWhitelist) && is_array($packagesWhitelist)) {
            $release = $this->utils->checkRelease();
            $installed = $this->listInstalled();
            if (file_exists('/tmp/modules-'.$release) && (time() - filemtime('/tmp/modules-'.$release)) < 600) {
                $modules = json_decode(file_get_contents('/tmp/modules-'.$release),true);
            } else {
                $ch = curl_init();
                curl_setopt($ch, CURLOPT_URL,'https://apt.smartha.io/list.php');
                curl_setopt($ch, CURLOPT_POST,1);
                curl_setopt($ch, CURLOPT_POSTFIELDS,"release=$release");
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
                $data = curl_exec($ch);
                $response = curl_getinfo($ch,CURLINFO_HTTP_CODE);
                curl_close($ch);
                if ($response == '200') {
                    $file = fopen('/tmp/modules-'.$release, "w");
                    fputs($file, $data);
                    fclose($file);
                    $modules = json_decode(file_get_contents('/tmp/modules-'.$release),true);
                }
            }
            foreach ($modules as $mkey => $module) {
                if (!in_array($module['package'], $packagesWhitelist)) { 
                    unset($modules[$mkey]);
                    continue;
                }
                if ($module['name'] == 'Miscellaneous') {
                    $miscInstalled = shell_exec("dpkg -l ".$module['package']);
                    if ($miscInstalled && $miscInstalled != null) {
                        $modules[$mkey]['installed'] = true;
                    }
                } else {
                    $key = array_search($module['file'], array_column($installed, 'FILENAME'));
                    if ($key !== false) {
                        $modules[$mkey]['installed'] = true;
                    }
                }
            }
        }
        return $modules;
    }

    /**
     * Load and activate a module
     * 
     * @return array()
     */
    public function load($module)
    {
        if ($module === null) { return; }
        $config = explode("_",$module,2);
        $config = preg_replace('/.so$/','',$config[1]);
        if (class_exists('App\Model\Admin\SmarthaLicense')) {
            $myLic = new SmarthaLicense();
            if ($myLic->checkLoad($config) === false) {
                return ['result' => 'error', 'message' => $this->translator->trans("ajax.modules.load.result_-4",array(),'admin')];
            }
        }
        if ($this->utils->checkManagementRunning(false)) {
            try {
                if ($config == 'zwave') {
                    $this->hg->managementSetConfigurationEntry("families/z-wave.conf","moduleEnabled","true");
                } elseif ($config == 'mbus') {
                    $this->hg->managementSetConfigurationEntry("families/m-bus.conf","moduleEnabled","true");
                } elseif ($config == 'velux_klf200') {
                    $this->hg->managementSetConfigurationEntry("families/veluxklf200.conf","moduleEnabled","true");
                } else {
                    $this->hg->managementSetConfigurationEntry("families/$config.conf","moduleEnabled","true");
                }
            } catch(\Homegear\HomegearException $e) {
                return ['result' => 'error', 'message' => $e->getMessage()];
            }
        }
        try {
            $result = $this->hg->loadModule('mod_'.$config.'.so');
            return ['result' => ($result != 0 ? 'error' : 'success'), 'message' => $this->translator->trans("ajax.modules.load.result_$result",array(),'admin')];
        } catch(\Homegear\HomegearException $e) {
            return ['result' => 'error', 'message' => $e->getMessage()];
        }
    }

    /**
     * Unload and deactivate a module
     * 
     * @return bool
     */
    public function unload($module = null)
    {
        if ($module === null) { return; }
        $config = explode("_",$module,2);
        $config = preg_replace('/.so$/','',$config[1]);
        if ($this->utils->checkManagementRunning(false)) {
            try {
                if ($config == 'zwave') {
                    $this->hg->managementSetConfigurationEntry("families/z-wave.conf","moduleEnabled","false");
                } elseif ($config == 'mbus') {
                    $this->hg->managementSetConfigurationEntry("families/m-bus.conf","moduleEnabled","false");
                } elseif ($config == 'velux_klf200') {
                    $this->hg->managementSetConfigurationEntry("families/veluxklf200.conf","moduleEnabled","false");
                } else {
                    $this->hg->managementSetConfigurationEntry("families/$config.conf","moduleEnabled","false");
                }
            } catch(\Homegear\HomegearException $e) {
                return ['result' => 'error', 'message' => $e->getMessage()];
            }
        }
        try {
            $result = $this->hg->unloadModule($module);
            return ['result' => ($result != 0 ? 'error' : 'success'), 'message' => $this->translator->trans("ajax.modules.unload.result_$result",array(),'admin')];
        } catch(\Homegear\HomegearException $e) {
            return ['result' => 'error', 'message' => $e->getMessage()];
        }
    }

    /**
     * reload and deactivate a module
     * 
     * @return bool
     */
    public function reload($module = null)
    {
        if ($module === null) { return; }
        try {
            $result = $this->hg->reloadModule($module);
            return ['result' => ($result != 0 ? 'error' : 'success'), 'message' => $this->translator->trans("ajax.modules.reload.result_$result",array(),'admin')];
        } catch(\Homegear\HomegearException $e) {
            return ['result' => 'error', 'message' => $e->getMessage()];
        }
    }

    /**
     * install a module
     * 
     * @return json
     */
    public function install($package = null)
    {
        if ($package === null) { return; }
        if ($this->utils->checkSmarthaManagementRunning(false)) {
            try {
                // do a apt update first, maybe the list on our server is newer than the apt database
                $aptId = $this->hg->managementAptUpdate();
                $result = $this->utils->waitAptFinished();
                if ($result === true) {
                    $id = $this->hg->smarthaAptInstall($package);
                    return ['result' => 'success', 'id' => $id];
                }
                return ['result' => 'error', 'msg' => $this->translator->trans('updates.error_checking',array(),'admin')];
            } catch(\Homegear\HomegearException $e) {
                return ['result' => 'error', 'message' => $e->getMessage()];
            }
        }
    }

    public function checkInstalling($id = null)
    {
        if ($id === null) { return; }
        $result = true;
        try {
            $result = $this->hg->managementAptRunning();
        } catch(\Homegear\HomegearException $e) {
            return ['result' => 'error', 'message' => $e->getMessage()];
        }
        if ($result !== false) {
            // apt still running
            return ['result' => 'true'];
        }
        // apt finished
        return ['result' => 'false'];
    }

    /**
     * Check module license
     * 
     * @return int
     */
    public function checkLicense($family = null, $key = null, $module = null) {
        if ($family === null || $key === null) { return; }
        if ($module === null) {
            $list = $this->hg->listModules();
            foreach ($list as $mod) {
            if ($mod['FAMILY_ID'] == $family) {
                $module = $mod['FILENAME'];
              }
            }
        }
        if (!isset($module)) { return; }
        $return_code = $this->hg->checkLicense((int)hexdec('0x7000'),(int)$family,-1,$key);
        if (class_exists('App\Model\Admin\SmarthaLicense')) {
            $myLic = new SmarthaLicense();
            $myLic->checkLicense($family, $key, $return_code);
        }
        if ($return_code >= 0) {
            // reload module
            $this->reload($module);
        } else {
            $this->unload($module);
        }
        return $return_code;
    }
}